package token;

import java.util.function.Supplier;

import file.FilePosition;
import sexpression.Cons;
import sexpression.SExpression;

/**
 * A token in Lisp.
 */
public abstract class Token {

    private String text;
    private FilePosition position;

    public Token(String text, FilePosition position) {
        this.text = text;
        this.position = position;
    }

    public String getText() {
        return text;
    }

    public String getFileName() {
        return position.getFileName();
    }

    public int getLine() {
        return position.getLineNumber();
    }

    public int getColumn() {
        return position.getColumnNumber();
    }

    // sExpr ::= NUMBER | IDENTIFIER | STRING | QUOTE_MARK sExpr |
    // LEFT_PAREN sExprTail
    public abstract SExpression sExpr(Supplier<Token> getNextToken);

    // sExprTail ::= RIGHT_PAREN | sExpr sExprTail
    public SExpression sExprTail(Supplier<Token> getNextToken) {
        SExpression car = sExpr(getNextToken);

        Token nextToken = getNextToken.get();
        SExpression cdr = nextToken.sExprTail(getNextToken);

        return new Cons(car, cdr);
    }

}
